<?php

namespace Drupal\inspect_test\Commands;

use Drush\Commands\DrushCommands;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\inspect\InspectHelperConfig;

/**
 * Inspect Test drush commands.
 *
 * CLI:
 * vendor/bin/drush inspect_test:default
 */
class InspectTestCommands extends DrushCommands {
  /**
   * Super module (Inspect) configuration.
   *
   * @var InspectHelperConfig
   */
  protected $superModuleConfig;

  /**
   * @param ConfigFactoryInterface $config_factory
   */
  public function __construct(
    ConfigFactoryInterface $config_factory
  ) {
    $this->superModuleConfig = new InspectHelperConfig(
      $config_factory->get('inspect.settings')
    );

    parent::__construct();
  }

  /**
   * CLI:
   * vendor/bin/drush inspect_test:default
   *
   * @command inspect_test:default
   */
  public function default() {
    /** @var \Drupal\inspect\Inspect $inspect */
    $inspect = \Drupal::service('inspect.inspect');

    // Variable inspection.
    /** @var \Drupal\inspect\Inspector $inspector */
    $inspector = $inspect->variable($this->superModuleConfig);
    $msg = 'inspect.settings:';
    $inspector->log('debug', $msg, ['channel' => 'inspect_test']);
    //$this->io()->note(
    $this->output()->write(
      $msg . "\n" . $inspector,
      TRUE
    );

    $inspector = $inspect->variable($GLOBALS);
    $msg = '$GLOBALS:';
    $inspector->log('debug', $msg, ['channel' => 'inspect_test']);
    $this->output()->write(
      "\n" . $msg . "\n" . $inspector,
      TRUE
    );

    $inspector = $inspect->variable($inspector);
    $msg = 'inspector self:';
    $inspector->log('debug', $msg, ['channel' => 'inspect_test']);
    $this->output()->write(
      "\n" . $msg . "\n" . $inspector,
      TRUE
    );

    // Backtrace.
    $inspector = $inspect->trace(NULL);
    $msg = 'backtrace:';
    $inspector->log('debug', $msg, ['channel' => 'inspect_test']);
    $this->output()->write(
      "\n" . $msg . "\n" . $inspector,
      TRUE
    );

    // Exception trace.
    try {
      throw new \RuntimeException('Doh!');
    }
    catch (\Throwable $xcptn) {
      $inspector = $inspect->trace($xcptn, ['limit' => 2]);
      $msg = 'exception:';
      $inspector->log('debug', $msg, ['channel' => 'inspect_test']);
      $this->output()->write(
        "\n" . $msg . "\n" . $inspector,
        TRUE
      );
    }

    // Shorthand functions.
    inspect($this->superModuleConfig, [
      'channel' => 'inspect_test',
      'message' => 'inspect(inspect.settings):',
    ]);
    inspect_trace(NULL, [
      'channel' => 'inspect_test',
      'message' => 'inspect_trace(NULL):',
    ]);
    // Shorthand non-array options.
    inspect($this->superModuleConfig, 2);
    inspect($this->superModuleConfig, (object) ['message' => 'inspect(..., object)']);
    inspect($this->superModuleConfig, 'inspect(..., message)');
    inspect(NULL, 'trace');
    inspect_trace(NULL, 3);

    return 0;
  }

}

